@@ -11,17 +11,13 @@ import shutil
 import subprocess
 import sys
 import typing as t
-from typing import Dict, Generic, KeysView, List, Optional, TypeVar, Union, overload
+from typing import Dict, Optional, Union
 
 from . import exc
 from ._compat import LooseVersion, console_to_str, str_from_console
 
 if t.TYPE_CHECKING:
-    from typing_extensions import Literal
-
-    from libtmux.pane import Pane
-    from libtmux.session import Session
-    from libtmux.window import Window
+    pass
 
 
 logger = logging.getLogger(__name__)
@@ -73,7 +69,7 @@ class EnvironmentMixin:
         cmd = self.cmd(*args)
 
         if cmd.stderr:
-            stderr = (
+            (
                 cmd.stderr[0]
                 if isinstance(cmd.stderr, list) and len(cmd.stderr) == 1
                 else cmd.stderr
@@ -97,7 +93,7 @@ class EnvironmentMixin:
         cmd = self.cmd(*args)
 
         if cmd.stderr:
-            stderr = (
+            (
                 cmd.stderr[0]
                 if isinstance(cmd.stderr, list) and len(cmd.stderr) == 1
                 else cmd.stderr
@@ -120,7 +116,7 @@ class EnvironmentMixin:
         cmd = self.cmd(*args)
 
         if cmd.stderr:
-            stderr = (
+            (
                 cmd.stderr[0]
                 if isinstance(cmd.stderr, list) and len(cmd.stderr) == 1
                 else cmd.stderr
@@ -149,13 +145,13 @@ class EnvironmentMixin:
         output = cmd.stdout
         vars = [tuple(item.split("=", 1)) for item in output]
         vars_dict: t.Dict[str, t.Union[str, bool]] = {}
-        for t in vars:
-            if len(t) == 2:
-                vars_dict[t[0]] = t[1]
-            elif len(t) == 1:
-                vars_dict[t[0]] = True
+        for _t in vars:
+            if len(_t) == 2:
+                vars_dict[_t[0]] = _t[1]
+            elif len(_t) == 1:
+                vars_dict[_t[0]] = True
             else:
-                raise ValueError("unexpected variable %s", t)
+                raise exc.VariableUnpackingError(variable=_t)
 
         return vars_dict
 
@@ -176,7 +172,7 @@ class EnvironmentMixin:
         str
             Value of environment variable
         """
-        tmux_args: t.Tuple[t.Union[str, int], ...] = tuple()
+        tmux_args: t.Tuple[t.Union[str, int], ...] = ()
 
         tmux_args += ("show-environment",)
         if self._add_option:
@@ -186,13 +182,13 @@ class EnvironmentMixin:
         output = cmd.stdout
         vars = [tuple(item.split("=", 1)) for item in output]
         vars_dict: t.Dict[str, t.Union[str, bool]] = {}
-        for t in vars:
-            if len(t) == 2:
-                vars_dict[t[0]] = t[1]
-            elif len(t) == 1:
-                vars_dict[t[0]] = True
+        for _t in vars:
+            if len(_t) == 2:
+                vars_dict[_t[0]] = _t[1]
+            elif len(_t) == 1:
+                vars_dict[_t[0]] = True
             else:
-                raise ValueError("unexpected variable %s", t)
+                raise exc.VariableUnpackingError(variable=_t)
 
         return vars_dict.get(name)
 
@@ -246,8 +242,8 @@ class tmux_cmd:
             )
             stdout, stderr = self.process.communicate()
             returncode = self.process.returncode
-        except Exception as e:
-            logger.error(f"Exception for {subprocess.list2cmdline(cmd)}: \n{e}")
+        except Exception:
+            logger.exception(f"Exception for {subprocess.list2cmdline(cmd)}")
             raise
 
         self.returncode = returncode
@@ -267,175 +263,11 @@ class tmux_cmd:
         else:
             self.stdout = stdout_split
 
-        logger.debug("self.stdout for {}: \n{}".format(" ".join(cmd), self.stdout))
-
-
-# class TmuxMappingObject(t.Mapping[str, t.Union[str,int,bool]]):
-class TmuxMappingObject(t.Mapping[t.Any, t.Any]):
-    r"""Base: :py:class:`MutableMapping`.
-
-    Convenience container. Base class for :class:`Pane`, :class:`Window`,
-    :class:`Session` and :class:`Server`.
-
-    Instance attributes for useful information :term:`tmux(1)` uses for
-    Session, Window, Pane, stored :attr:`self._info`. For example, a
-    :class:`Window` will have a ``window_id`` and ``window_name``.
-
-    ================ ================================== ==============
-    Object           formatter_prefix                   value
-    ================ ================================== ==============
-    :class:`Server`  n/a                                n/a
-    :class:`Session` :attr:`Session.formatter_prefix`   session\_
-    :class:`Window`  :attr:`Window.formatter_prefix`    window\_
-    :class:`Pane`    :attr:`Pane.formatter_prefix`      pane\_
-    ================ ================================== ==============
-    """
-    _info: t.Dict[t.Any, t.Any]
-    formatter_prefix: str
-
-    def __getitem__(self, key: str) -> str:
-        item = self._info[key]
-        assert item is not None
-        assert isinstance(item, str)
-        return item
-
-    def __setitem__(self, key: str, value: str) -> None:
-        self._info[key] = value
-        self.dirty = True
-
-    def __delitem__(self, key: str) -> None:
-        del self._info[key]
-        self.dirty = True
-
-    def keys(self) -> KeysView[str]:
-        """Return list of keys."""
-        return self._info.keys()
-
-    def __iter__(self) -> t.Iterator[str]:
-        return self._info.__iter__()
-
-    def __len__(self) -> int:
-        return len(self._info.keys())
-
-    def __getattr__(self, key: str) -> str:
-        try:
-            val = self._info[self.formatter_prefix + key]
-            assert val is not None
-            assert isinstance(val, str)
-            return val
-        except KeyError:
-            raise AttributeError(f"{self.__class__} has no property {key}")
-
-
-O = TypeVar("O", "Pane", "Window", "Session")
-D = TypeVar("D", "PaneDict", "WindowDict", "SessionDict")
-
-
-class TmuxRelationalObject(Generic[O, D]):
-    """Base Class for managing tmux object child entities.  .. # NOQA
-
-    Manages collection of child objects  (a :class:`Server` has a collection of
-    :class:`Session` objects, a :class:`Session` has collection of
-    :class:`Window`)
-
-    Children of :class:`TmuxRelationalObject` are going to have a
-    ``self.children``, ``self.child_id_attribute``.
-
-    ================ ========================= =================================
-    Object           .children                 method
-    ================ ========================= =================================
-    :class:`Server`  :attr:`Server._sessions`  :meth:`Server.list_sessions`
-    :class:`Session` :attr:`Session._windows`  :meth:`Session.list_windows`
-    :class:`Window`  :attr:`Window._panes`     :meth:`Window.list_panes`
-    :class:`Pane`    n/a                       n/a
-    ================ ========================= =================================
-
-    ================ ================================== ==============
-    Object           child_id_attribute                 value
-    ================ ================================== ==============
-    :class:`Server`  :attr:`Server.child_id_attribute`  session_id
-    :class:`Session` :attr:`Session.child_id_attribute` window_id
-    :class:`Window`  :attr:`Window.child_id_attribute`  pane_id
-    :class:`Pane`    n/a                                n/a
-    ================ ================================== ==============
-    """
-
-    children: t.List[O]
-    child_id_attribute: str
-
-    def find_where(self, attrs: D) -> Optional[Union["Pane", "Window", "Session"]]:
-        """Return object on first match.
-
-        .. versionchanged:: 0.4
-            Renamed from ``.findWhere`` to ``.find_where``.
-
-        """
-        try:
-            return self.where(attrs)[0]
-        except IndexError:
-            return None
-
-    @overload
-    def where(self, attrs: D, first: "Literal[True]") -> O:
-        ...
-
-    @overload
-    def where(self, attrs: D, first: "Literal[False]") -> t.List[O]:
-        ...
-
-    @overload
-    def where(self, attrs: D) -> t.List[O]:
-        ...
-
-    def where(self, attrs: D, first: bool = False) -> t.Union[List[O], O]:
-        """
-        Return objects matching child objects properties.
-
-        Parameters
-        ----------
-        attrs : dict
-            tmux properties to match values of
-
-        Returns
-        -------
-        list of objects, or one object if ``first=True``
-        """
-
-        # from https://github.com/serkanyersen/underscore.py
-        def by(val: O) -> bool:
-            for key in attrs.keys():
-                try:
-                    if attrs[key] != val[key]:
-                        return False
-                except KeyError:
-                    return False
-            return True
-
-        target_children: t.List[O] = [s for s in self.children if by(s)]
-
-        if first:
-            return target_children[0]
-        return target_children
-
-    def get_by_id(self, id: str) -> Optional[O]:
-        """
-        Return object based on ``child_id_attribute``.
-
-        Parameters
-        ----------
-        val : str
-
-        Returns
-        -------
-        object
-        """
-        for child in self.children:
-            if child[self.child_id_attribute] == id:
-                return child
-            else:
-                continue
-
-        return None
+        logger.debug(
+            "self.stdout for {cmd}: {stdout}".format(
+                cmd=" ".join(cmd), stdout=self.stdout
+            )
+        )
 
 
 def get_version() -> LooseVersion:
@@ -593,9 +425,10 @@ def has_minimum_version(raises: bool = True) -> bool:
     if get_version() < LooseVersion(TMUX_MIN_VERSION):
         if raises:
             raise exc.VersionTooLow(
-                "libtmux only supports tmux %s and greater. This system"
-                " has %s installed. Upgrade your tmux to use libtmux."
-                % (TMUX_MIN_VERSION, get_version())
+                "libtmux only supports tmux {} and greater. This system"
+                " has {} installed. Upgrade your tmux to use libtmux.".format(
+                    TMUX_MIN_VERSION, get_version()
+                )
             )
         else:
             return False
@@ -620,15 +453,11 @@ def session_check_name(session_name: t.Optional[str]) -> None:
         Invalid session name.
     """
     if session_name is None or len(session_name) == 0:
-        raise exc.BadSessionName("tmux session names may not be empty.")
+        raise exc.BadSessionName(reason="empty", session_name=session_name)
     elif "." in session_name:
-        raise exc.BadSessionName(
-            'tmux session name "%s" may not contain periods.', session_name
-        )
+        raise exc.BadSessionName(reason="contains periods", session_name=session_name)
     elif ":" in session_name:
-        raise exc.BadSessionName(
-            'tmux session name "%s" may not contain colons.', session_name
-        )
+        raise exc.BadSessionName(reason="contains colons", session_name=session_name)
 
 
 def handle_option_error(error: str) -> t.Type[exc.OptionError]:
