@@ -8,7 +8,7 @@ For reference: https://github.com/tmux/tmux/blob/master/format.c
 """
 import os
 
-FORMAT_SEPARATOR = os.environ.get("LIBTMUX_TMUX_FORMAT_SEPARATOR", "|")
+FORMAT_SEPARATOR = os.environ.get("LIBTMUX_TMUX_FORMAT_SEPARATOR", "␞")
 
 SESSION_FORMATS = [
     "session_name",
@@ -41,14 +41,12 @@ CLIENT_FORMATS = [
 ]
 
 WINDOW_FORMATS = [
-    # format_window()
     "window_id",
     "window_name",
     "window_width",
     "window_height",
     "window_layout",
     "window_panes",
-    # format_winlink()
     "window_index",
     "window_flags",
     "window_active",
