@@ -6,7 +6,7 @@ libtmux.session
 """
 import dataclasses
 import logging
-import os
+import pathlib
 import typing as t
 import warnings
 
@@ -34,42 +34,8 @@ if t.TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
-# Dataclasses define the schema in a typed annotation way.
-#
-# Users load through YAML, JSON, TOML, or any sort of data entry, these are parsed
-# into dataclasses. Bi-directionals.
-#
-# This means the "intermediary language" is actually a first class, typed, declarative
-# python syntax. It's so first class, it's used in tests, exposed as an experimental API
-# with full intention for users to bootstrap sessions fast.
-#
-# Pitfalls: Lock / pin your versions. APIs may change. It's a Public API - but
-# considered experimental in the sense we .
-#
-# The good news is, since it's typed, you will be able to refactor quickly. In fact,
-# it's easier to factor these than a config where there's no safety.
-#
-# Session.build_workspace(
-#     windows=[
-#         Window(
-#             panes=[
-#                 panes=[]
-#             ]
-#
-#         )
-#     ],
-#     settings=WorkspaceBuilderSettings(
-#         backend='libtmux.ext.workspace_builder.backends.default'
-#     )
-# )
-# WorkspaceBuilder:
-#     backend = 'libtmux.ext.workspace_builder.backends.default'
-#
-
-
 @dataclasses.dataclass()
 class Session(Obj, EnvironmentMixin):
-
     """
     A :term:`tmux(1)` :term:`Session` [session_manual]_.
 
@@ -135,27 +101,35 @@ class Session(Obj, EnvironmentMixin):
         :meth:`.windows.get() <libtmux._internal.query_list.QueryList.get()>` and
         :meth:`.windows.filter() <libtmux._internal.query_list.QueryList.filter()>`
         """
-        windows: t.List["Window"] = []
-        for obj in fetch_objs(
-            list_cmd="list-windows",
-            list_extra_args=["-t", str(self.session_id)],
-            server=self.server,
-        ):
-            if obj.get("session_id") == self.session_id:
-                windows.append(Window(server=self.server, **obj))
+        windows: t.List["Window"] = [
+            Window(server=self.server, **obj)
+            for obj in fetch_objs(
+                list_cmd="list-windows",
+                list_extra_args=["-t", str(self.session_id)],
+                server=self.server,
+            )
+            if obj.get("session_id") == self.session_id
+        ]
 
         return QueryList(windows)
 
     @property
     def panes(self) -> QueryList["Pane"]:  # type:ignore
-        panes: t.List["Pane"] = []
-        for obj in fetch_objs(
-            list_cmd="list-panes",
-            list_extra_args=["-s", "-t", str(self.session_id)],
-            server=self.server,
-        ):
-            if obj.get("session_id") == self.session_id:
-                panes.append(Pane(server=self.server, **obj))
+        """Panes belonging session.
+
+        Can be accessed via
+        :meth:`.panes.get() <libtmux._internal.query_list.QueryList.get()>` and
+        :meth:`.panes.filter() <libtmux._internal.query_list.QueryList.filter()>`
+        """
+        panes: t.List["Pane"] = [
+            Pane(server=self.server, **obj)
+            for obj in fetch_objs(
+                list_cmd="list-panes",
+                list_extra_args=["-s", "-t", str(self.session_id)],
+                server=self.server,
+            )
+            if obj.get("session_id") == self.session_id
+        ]
 
         return QueryList(panes)
 
@@ -179,7 +153,7 @@ class Session(Obj, EnvironmentMixin):
         # if -t is not set in any arg yet
         if not any("-t" in str(x) for x in args):
             # insert -t immediately after 1st arg, as per tmux format
-            new_args: t.Tuple[str, ...] = tuple()
+            new_args: t.Tuple[str, ...] = ()
             new_args += (args[0],)
             assert isinstance(self.session_id, str)
             new_args += (
@@ -191,7 +165,6 @@ class Session(Obj, EnvironmentMixin):
         return self.server.cmd(*args, **kwargs)
 
     #
-    # Commands (tmux-like)
     #
     def set_option(
         self, option: str, value: t.Union[str, int], _global: bool = False
@@ -225,7 +198,7 @@ class Session(Obj, EnvironmentMixin):
         elif isinstance(value, bool) and not value:
             value = "off"
 
-        tmux_args: t.Tuple[t.Union[str, int], ...] = tuple()
+        tmux_args: t.Tuple[t.Union[str, int], ...] = ()
 
         if _global:
             tmux_args += ("-g",)
@@ -268,7 +241,7 @@ class Session(Obj, EnvironmentMixin):
         Uses ``_global`` for keyword name instead of ``global`` to avoid
         colliding with reserved keyword.
         """
-        tmux_args: t.Tuple[str, ...] = tuple()
+        tmux_args: t.Tuple[str, ...] = ()
 
         if _global:
             tmux_args += ("-g",)
@@ -316,7 +289,7 @@ class Session(Obj, EnvironmentMixin):
         Test and return True/False for on/off string.
         """
 
-        tmux_args: t.Tuple[str, ...] = tuple()
+        tmux_args: t.Tuple[str, ...] = ()
 
         if _global:
             tmux_args += ("-g",)
@@ -331,7 +304,7 @@ class Session(Obj, EnvironmentMixin):
         if not len(cmd.stdout):
             return None
 
-        value_raw: t.List[str] = [item.split(" ") for item in cmd.stdout][0]
+        value_raw: t.List[str] = next(item.split(" ") for item in cmd.stdout)
 
         assert isinstance(value_raw[0], str)
         assert isinstance(value_raw[1], str)
@@ -383,23 +356,19 @@ class Session(Obj, EnvironmentMixin):
         """
         Return active :class:`Window` object.
         """
-        active_windows = []
-        for window in self.windows:
-            # for now window_active is a unicode
-            if window.window_active == "1":
-                active_windows.append(window)
+        active_windows = [
+            window for window in self.windows if window.window_active == "1"
+        ]
 
         if len(active_windows) == 1:
-            return list(active_windows)[0]
+            return next(iter(active_windows))
         elif len(active_windows) == 0:
-            raise exc.LibTmuxException("no active windows found")
+            raise exc.NoActiveWindow()
         else:
-            raise exc.LibTmuxException(
-                "multiple active windows found. %s" % active_windows
-            )
+            raise exc.MultipleActiveWindows(count=len(active_windows))
 
         if len(self._windows) == 0:
-            raise exc.LibTmuxException("No Windows")
+            raise exc.NoWindowsExist()
 
     def attach_session(self) -> "Session":
         """Return ``$ tmux attach-session`` aka alias: ``$ tmux attach``."""
@@ -504,7 +473,7 @@ class Session(Obj, EnvironmentMixin):
         -------
         :class:`Window`
         """
-        window_args: t.Tuple[str, ...] = tuple()
+        window_args: t.Tuple[str, ...] = ()
 
         if not attach:
             window_args += ("-d",)
@@ -513,8 +482,8 @@ class Session(Obj, EnvironmentMixin):
 
         if start_directory:
             # as of 2014-02-08 tmux 1.9-dev doesn't expand ~ in new-window -c.
-            start_directory = os.path.expanduser(start_directory)
-            window_args += ("-c%s" % start_directory,)
+            start_directory = pathlib.Path(start_directory).expanduser()
+            window_args += (f"-c{start_directory}",)
 
         window_args += ("-F#{window_id}",)  # output
         if window_name is not None and isinstance(window_name, str):
@@ -522,8 +491,7 @@ class Session(Obj, EnvironmentMixin):
 
         window_args += (
             # empty string for window_index will use the first one available
-            "-t%s:%s"
-            % (self.session_id, window_index),
+            f"-t{self.session_id}:{window_index}",
         )
 
         if environment:
@@ -629,28 +597,30 @@ class Session(Obj, EnvironmentMixin):
         """
         .. deprecated:: 0.16
         """
-        warnings.warn("Session.get() is deprecated")
+        warnings.warn("Session.get() is deprecated", stacklevel=2)
         return getattr(self, key, default)
 
     def __getitem__(self, key: str) -> t.Any:
         """
         .. deprecated:: 0.16
         """
-        warnings.warn(f"Item lookups, e.g. session['{key}'] is deprecated")
+        warnings.warn(
+            f"Item lookups, e.g. session['{key}'] is deprecated", stacklevel=2
+        )
         return getattr(self, key)
 
     def get_by_id(self, id: str) -> t.Optional[Window]:
         """
         .. deprecated:: 0.16
         """
-        warnings.warn("Session.get_by_id() is deprecated")
+        warnings.warn("Session.get_by_id() is deprecated", stacklevel=2)
         return self.windows.get(window_id=id, default=None)
 
     def where(self, kwargs: t.Dict[str, t.Any]) -> t.List[Window]:
         """
         .. deprecated:: 0.16
         """
-        warnings.warn("Session.where() is deprecated")
+        warnings.warn("Session.where() is deprecated", stacklevel=2)
         try:
             return self.windows.filter(**kwargs)
         except IndexError:
@@ -660,14 +630,14 @@ class Session(Obj, EnvironmentMixin):
         """
         .. deprecated:: 0.16
         """
-        warnings.warn("Session.find_where() is deprecated")
+        warnings.warn("Session.find_where() is deprecated", stacklevel=2)
         return self.windows.get(default=None, **kwargs)
 
     def _list_windows(self) -> t.List["WindowDict"]:
         """
         .. deprecated:: 0.16
         """
-        warnings.warn("Session._list_windows() is deprecated")
+        warnings.warn("Session._list_windows() is deprecated", stacklevel=2)
         return [w.__dict__ for w in self.windows]
 
     @property
@@ -676,7 +646,7 @@ class Session(Obj, EnvironmentMixin):
 
         .. deprecated:: 0.16
         """
-        warnings.warn("Session._windows is deprecated")
+        warnings.warn("Session._windows is deprecated", stacklevel=2)
         return self._list_windows()
 
     def list_windows(self) -> t.List["Window"]:
@@ -684,7 +654,7 @@ class Session(Obj, EnvironmentMixin):
 
         .. deprecated:: 0.16
         """
-        warnings.warn("Session.list_windows() is deprecated")
+        warnings.warn("Session.list_windows() is deprecated", stacklevel=2)
         return self.windows
 
     @property
@@ -693,5 +663,5 @@ class Session(Obj, EnvironmentMixin):
 
         .. deprecated:: 0.16
         """
-        warnings.warn("Session.children is deprecated")
+        warnings.warn("Session.children is deprecated", stacklevel=2)
         return self.windows
